@@ -71,6 +71,8 @@ gpointer	 gs_plugin_get_symbol			(GsPlugin	*plugin,
 							 const gchar	*function_name);
 gchar		*gs_plugin_failure_flags_to_string	(GsPluginFailureFlags failure_flags);
 gchar		*gs_plugin_refine_flags_to_string	(GsPluginRefineFlags refine_flags);
+void		 gs_plugin_set_network_monitor		(GsPlugin		*plugin,
+							 GNetworkMonitor	*monitor);
 
 G_END_DECLS
 
