@@ -83,6 +83,7 @@ typedef struct
 	guint			 priority;
 	guint			 timer_id;
 	GMutex			 timer_mutex;
+	GNetworkMonitor		*network_monitor;
 } GsPluginPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GsPlugin, gs_plugin, G_TYPE_OBJECT)
@@ -230,6 +231,8 @@ gs_plugin_finalize (GObject *object)
 		g_object_unref (priv->soup_session);
 	if (priv->global_cache != NULL)
 		g_object_unref (priv->global_cache);
+	if (priv->network_monitor != NULL)
+		g_object_unref (priv->network_monitor);
 	g_hash_table_unref (priv->cache);
 	g_hash_table_unref (priv->vfuncs);
 	g_mutex_clear (&priv->cache_mutex);
@@ -804,6 +807,43 @@ gs_plugin_set_global_cache (GsPlugin *plugin, GsAppList *global_cache)
 	g_set_object (&priv->global_cache, global_cache);
 }
 
+/**
+ * gs_plugin_set_network_monitor:
+ * @plugin: a #GsPlugin
+ * @network_monitor: a #GNetworkMonitor
+ *
+ * Sets the network monitor so that plugins can check the state of the network.
+ *
+ * Since: 3.28
+ **/
+void
+gs_plugin_set_network_monitor (GsPlugin *plugin, GNetworkMonitor *monitor)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	g_set_object (&priv->network_monitor, monitor);
+}
+
+/**
+ * gs_plugin_get_network_available:
+ * @plugin: a #GsPlugin
+ *
+ * Gets whether a network connectivity is available.
+ *
+ * Returns: %TRUE if a network is available.
+ *
+ * Since: 3.28
+ **/
+gboolean
+gs_plugin_get_network_available (GsPlugin *plugin)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	if (priv->network_monitor == NULL) {
+		g_debug ("no network monitor, so returning network-available=TRUE");
+		return TRUE;
+	}
+	return g_network_monitor_get_network_available (priv->network_monitor);
+}
+
 /**
  * gs_plugin_has_flags:
  * @plugin: a #GsPlugin
@@ -1463,11 +1503,13 @@ gs_plugin_cache_lookup (GsPlugin *plugin, const gchar *key)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
 	GsApp *app;
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->cache_mutex);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	g_return_val_if_fail (GS_IS_PLUGIN (plugin), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
+	locker = g_mutex_locker_new (&priv->cache_mutex);
+
 	/* global, so using a unique_id */
 	if (gs_plugin_has_flags (plugin, GS_PLUGIN_FLAGS_GLOBAL_CACHE)) {
 		if (!as_utils_unique_id_valid (key)) {
@@ -1496,11 +1538,13 @@ void
 gs_plugin_cache_remove (GsPlugin *plugin, const gchar *key)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->cache_mutex);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	g_return_if_fail (GS_IS_PLUGIN (plugin));
 	g_return_if_fail (key != NULL);
 
+	locker = g_mutex_locker_new (&priv->cache_mutex);
+
 	/* global, so using internal unique_id */
 	if (gs_plugin_has_flags (plugin, GS_PLUGIN_FLAGS_GLOBAL_CACHE)) {
 		GsApp *app_tmp;
@@ -1531,11 +1575,13 @@ void
 gs_plugin_cache_add (GsPlugin *plugin, const gchar *key, GsApp *app)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->cache_mutex);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	g_return_if_fail (GS_IS_PLUGIN (plugin));
 	g_return_if_fail (GS_IS_APP (app));
 
+	locker = g_mutex_locker_new (&priv->cache_mutex);
+
 	/* default */
 	if (key == NULL)
 		key = gs_app_get_unique_id (app);
@@ -1575,10 +1621,11 @@ void
 gs_plugin_cache_invalidate (GsPlugin *plugin)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->cache_mutex);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	g_return_if_fail (GS_IS_PLUGIN (plugin));
 
+	locker = g_mutex_locker_new (&priv->cache_mutex);
 	g_hash_table_remove_all (priv->cache);
 }
 
