@@ -1185,12 +1185,20 @@ gs_plugin_fwupd_modify_source (GsPlugin *plugin, GsApp *app, gboolean enabled,
 			     gs_app_get_unique_id (app));
 		return FALSE;
 	}
-	return fwupd_client_modify_remote (priv->client,
-					   remote_id,
-					   "Enabled",
-					   enabled ? "true" : "false",
-					   cancellable,
-					   error);
+	gs_app_set_state (app, enabled ?
+	                  AS_APP_STATE_INSTALLING : AS_APP_STATE_REMOVING);
+	if (!fwupd_client_modify_remote (priv->client,
+	                                 remote_id,
+	                                 "Enabled",
+	                                 enabled ? "true" : "false",
+	                                 cancellable,
+	                                 error)) {
+		gs_app_set_state_recover (app);
+		return FALSE;
+	}
+	gs_app_set_state (app, enabled ?
+	                  AS_APP_STATE_INSTALLED : AS_APP_STATE_AVAILABLE);
+	return TRUE;
 }
 
 gboolean
@@ -1224,7 +1232,7 @@ gs_plugin_app_remove (GsPlugin *plugin, GsApp *app,
 		return TRUE;
 
 	/* source -> remote */
-	return gs_plugin_fwupd_modify_source (plugin, app, TRUE, cancellable, error);
+	return gs_plugin_fwupd_modify_source (plugin, app, FALSE, cancellable, error);
 }
 
 gboolean
@@ -1379,6 +1387,7 @@ gs_plugin_add_sources (GsPlugin *plugin,
 				fwupd_remote_get_metadata_uri (remote));
 		gs_app_set_metadata (app, "fwupd::remote-id",
 				     fwupd_remote_get_id (remote));
+		gs_app_set_management_plugin (app, "fwupd");
 		gs_app_list_add (list, app);
 	}
 	return TRUE;
