@@ -422,8 +422,7 @@ gs_plugin_packagekit_set_metadata_from_package (GsPlugin *plugin,
 		gs_app_set_size_download (app, GS_APP_SIZE_UNKNOWABLE);
 	}
 	if (gs_app_get_version (app) == NULL)
-		gs_app_set_version (app,
-			pk_package_get_version (package));
+		gs_app_set_version (app, pk_package_get_version (package));
 	gs_app_set_name (app,
 			 GS_APP_QUALITY_LOWEST,
 			 pk_package_get_name (package));
@@ -497,6 +496,11 @@ gs_plugin_packagekit_refine_details_app (GsPlugin *plugin,
 						AS_URL_KIND_HOMEPAGE,
 						pk_details_get_url (details));
 			}
+			if (gs_app_get_description (app) == NULL) {
+				gs_app_set_description (app,
+				                        GS_APP_QUALITY_LOWEST,
+				                        pk_details_get_description (details));
+			}
 			size += pk_details_get_size (details);
 			break;
 		}
