@@ -47,13 +47,12 @@
 #include "gs-shell-search-provider.h"
 #include "gs-folders.h"
 
-#define ENABLE_SOFTWARE_SOURCES_CONF_KEY "enable-software-sources"
+#define ENABLE_REPOS_DIALOG_CONF_KEY "enable-repos-dialog"
 
 struct _GsApplication {
 	GtkApplication	 parent;
 	gboolean	 enable_profile_mode;
 	GCancellable	*cancellable;
-	GtkApplication	*application;
 	GtkCssProvider	*provider;
 	GsPluginLoader	*plugin_loader;
 	gint		 pending_apps;
@@ -250,6 +249,7 @@ gs_application_shell_loaded_cb (GsShell *shell, GsApplication *app)
 {
 	gs_shell_set_mode (app->shell, GS_SHELL_MODE_OVERVIEW);
 	app->shell_loaded_handler_id = 0;
+	gs_application_show_first_run_dialog (app);
 }
 
 static void
@@ -836,7 +836,7 @@ gs_application_update_software_sources_presence (GApplication *self)
 	action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (self),
 							      "sources"));
 	enable_sources = g_settings_get_boolean (app->settings,
-						 ENABLE_SOFTWARE_SOURCES_CONF_KEY);
+						 ENABLE_REPOS_DIALOG_CONF_KEY);
 	g_simple_action_set_enabled (action, enable_sources);
 }
 
@@ -845,7 +845,7 @@ gs_application_settings_changed_cb (GApplication *self,
 				    const gchar *key,
 				    gpointer data)
 {
-	if (g_strcmp0 (key, ENABLE_SOFTWARE_SOURCES_CONF_KEY) == 0) {
+	if (g_strcmp0 (key, ENABLE_REPOS_DIALOG_CONF_KEY) == 0) {
 		gs_application_update_software_sources_presence (self);
 	}
 }
@@ -959,8 +959,6 @@ gs_application_activate (GApplication *application)
 		gs_shell_set_mode (app->shell, GS_SHELL_MODE_OVERVIEW);
 
 	gs_shell_activate (GS_APPLICATION (application)->shell);
-
-	gs_application_show_first_run_dialog (GS_APPLICATION (application));
 }
 
 static void
