@@ -187,6 +187,8 @@ gs_category_page_sort_flow_box_sort_func (GtkFlowBoxChild *child1,
 	GsApp *app1 = gs_app_tile_get_app (GS_APP_TILE (gtk_bin_get_child (GTK_BIN (child1))));
 	GsApp *app2 = gs_app_tile_get_app (GS_APP_TILE (gtk_bin_get_child (GTK_BIN (child2))));
 	SubcategorySortType sort_type;
+	g_autofree gchar *casefolded_name1 = NULL;
+	g_autofree gchar *casefolded_name2 = NULL;
 
 	if (!GS_IS_APP (app1) || !GS_IS_APP (app2))
 		return 0;
@@ -201,7 +203,12 @@ gs_category_page_sort_flow_box_sort_func (GtkFlowBoxChild *child1,
 		if (rating_app1 < rating_app2)
 			return 1;
 	}
-	return g_strcmp0 (gs_app_get_name (app1), gs_app_get_name (app2));
+
+	if (gs_app_get_name (app1) != NULL)
+		casefolded_name1 = g_utf8_casefold (gs_app_get_name (app1), -1);
+	if (gs_app_get_name (app2) != NULL)
+		casefolded_name2 = g_utf8_casefold (gs_app_get_name (app2), -1);
+	return g_strcmp0 (casefolded_name1, casefolded_name2);
 }
 
 static void
@@ -473,6 +480,8 @@ gs_category_page_create_filter (GsCategoryPage *self,
 void
 gs_category_page_set_category (GsCategoryPage *self, GsCategory *category)
 {
+	GtkAdjustment *adj = NULL;
+
 	/* this means we've come from the app-view -> back */
 	if (self->category == category)
 		return;
@@ -483,6 +492,11 @@ gs_category_page_set_category (GsCategoryPage *self, GsCategory *category)
 
 	/* find apps in this group */
 	gs_category_page_create_filter (self, category);
+
+	/* scroll the list of apps to the beginning, otherwise it will show
+	 * with the previous scroll value */
+	adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self->scrolledwindow_category));
+	gtk_adjustment_set_value (adj, gtk_adjustment_get_lower (adj));
 }
 
 GsCategory *
