@@ -41,6 +41,7 @@ struct _GsInstalledPage
 	GCancellable		*cancellable;
 	GtkSizeGroup		*sizegroup_image;
 	GtkSizeGroup		*sizegroup_name;
+	GtkSizeGroup		*sizegroup_desc;
 	GtkSizeGroup		*sizegroup_button;
 	gboolean		 cache_valid;
 	gboolean		 waiting;
@@ -200,6 +201,7 @@ gs_installed_page_add_app (GsInstalledPage *self, GsAppList *list, GsApp *app)
 	gs_app_row_set_size_groups (GS_APP_ROW (app_row),
 				    self->sizegroup_image,
 				    self->sizegroup_name,
+				    self->sizegroup_desc,
 				    self->sizegroup_button);
 
 	if (!gs_app_has_quirk (app, AS_APP_QUIRK_COMPULSORY)) {
@@ -841,6 +843,7 @@ gs_installed_page_dispose (GObject *object)
 
 	g_clear_object (&self->sizegroup_image);
 	g_clear_object (&self->sizegroup_name);
+	g_clear_object (&self->sizegroup_desc);
 	g_clear_object (&self->sizegroup_button);
 
 	g_clear_object (&self->builder);
@@ -883,6 +886,7 @@ gs_installed_page_init (GsInstalledPage *self)
 
 	self->sizegroup_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_name = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	self->sizegroup_desc = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_button = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	self->settings = g_settings_new ("org.gnome.software");
