@@ -50,6 +50,8 @@ struct _GsPageClass
 						 GError		**error);
 };
 
+typedef void (*GsPageAuthCallback) (GsPage *page, gboolean authorized, gpointer user_data);
+
 GsPage		*gs_page_new				(void);
 GtkWidget	*gs_page_get_header_start_widget	(GsPage		*page);
 void		 gs_page_set_header_start_widget	(GsPage		*page,
@@ -57,6 +59,12 @@ void		 gs_page_set_header_start_widget	(GsPage		*page,
 GtkWidget	*gs_page_get_header_end_widget		(GsPage		*page);
 void		 gs_page_set_header_end_widget		(GsPage		*page,
 							 GtkWidget	*widget);
+void		 gs_page_authenticate			(GsPage			*page,
+							 GsApp			*app,
+							 const gchar		*provider_id,
+							 GCancellable		*cancellable,
+							 GsPageAuthCallback	 callback,
+							 gpointer		 user_data);
 void		 gs_page_install_app			(GsPage			*page,
 							 GsApp			*app,
 							 GsShellInteraction	interaction,
