@@ -36,7 +36,7 @@
 #include "gs-updates-page.h"
 #include "gs-category-page.h"
 #include "gs-extras-page.h"
-#include "gs-sources-dialog.h"
+#include "gs-repos-dialog.h"
 #include "gs-update-dialog.h"
 #include "gs-update-monitor.h"
 #include "gs-utils.h"
@@ -661,7 +661,7 @@ window_key_press_event (GtkWidget *win, GdkEventKey *event, GsShell *shell)
 	    	return GDK_EVENT_PROPAGATE;
 
 	state = event->state;
-	keymap = gdk_keymap_get_default ();
+	keymap = gdk_keymap_get_for_display (gtk_widget_get_display (win));
 	gdk_keymap_add_virtual_modifiers (keymap, &state);
 	state = state & gtk_accelerator_get_default_mod_mask ();
 	is_rtl = gtk_widget_get_direction (button) == GTK_TEXT_DIR_RTL;
@@ -1666,9 +1666,9 @@ gs_shell_events_notify_cb (GsPluginLoader *plugin_loader,
 static void
 gs_shell_plugin_event_dismissed_cb (GtkButton *button, GsShell *shell)
 {
-	GPtrArray *events;
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 	guint i;
+	g_autoptr(GPtrArray) events = NULL;
 
 	/* mark any events currently showing as invalid */
 	events = gs_plugin_loader_get_events (priv->plugin_loader);
@@ -1916,7 +1916,7 @@ gs_shell_show_sources (GsShell *shell)
 	if (g_spawn_command_line_async ("software-properties-gtk", NULL))
 		return;
 
-	dialog = gs_sources_dialog_new (priv->main_window, priv->plugin_loader);
+	dialog = gs_repos_dialog_new (priv->main_window, priv->plugin_loader);
 	gs_shell_modal_dialog_present (shell, GTK_DIALOG (dialog));
 
 	/* just destroy */
